SUBROUTINE dqp_range_kutta(po,qo,shrpct,dshrpct,jp,jq,lp,lq,ap,aq,pf,qf)
!
! This subroutine does Runge-Kutta 4th order integration
! From: http://www.physicsforums.com/showthread.php?t=508942
!
implicit none 
INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
real(iwp), intent(in)::po,qo,shrpct,dshrpct,jp,jq,lp,lq,ap,aq
real(iwp), intent(out)::pf,qf
real(iwp)::pi,kp1,kp2,kp3,kp4,kq1,kq2,kq3,kq4,dp,dq,one=1.0_iwp
pi=ACOS(-one)

kp1= po*tan(ap/180*pi)*jp*EXP(-lp*shrpct)-qo*jp*EXP(-lq*shrpct)
kq1= po*tan(aq/180*pi)*jq*EXP(-lp*shrpct)-qo*jq*EXP(-lq*shrpct)

kp2= (po+kp1/2*dshrpct)*tan(ap/180*pi)*jp*EXP(-lp*(shrpct+0.5*dshrpct)) &
	-(qo+kq1/2*dshrpct)*jp*EXP(-lq*(shrpct+0.5*dshrpct))
kq2= (po+kp1/2*dshrpct)*tan(aq/180*pi)*jq*EXP(-lp*(shrpct+0.5*dshrpct)) &
	-(qo+kq1/2*dshrpct)*jq*EXP(-lq*shrpct)

kp3= (po+kp2/2*dshrpct)*tan(ap/180*pi)*jp*EXP(-lp*(shrpct+0.5*dshrpct)) &
	-(qo+kq2/2*dshrpct)*jp*EXP(-lq*(shrpct+0.5*dshrpct))
kq3= (po+kp2/2*dshrpct)*tan(aq/180*pi)*jq*EXP(-lp*(shrpct+0.5*dshrpct)) &
	-(qo+kq2/2*dshrpct)*jq*EXP(-lq*shrpct)

kp4= (po+kp3*dshrpct)*tan(ap/180*pi)*jp*EXP(-lp*shrpct) &
	-(qo+kq3*dshrpct)*jp*EXP(-lq*shrpct)
kq4= (po+kp3*dshrpct)*tan(aq/180*pi)*jq*EXP(-lp*shrpct) &
	-(qo+kq3*dshrpct)*jq*EXP(-lq*shrpct)

dp=dshrpct*(kp1/3+kp2/6+kp3/6+kp4/3)
dq=dshrpct*(kq1/3+kq2/6+kq3/6+kq4/3)

!write(*,*)'po,qo,shrpct,dshrpct',po,qo,shrpct,dshrpct
!write(*,*)'jp,jq,lp,lq,ap,aq',jp,jq,lp,lq,ap,aq
!write(*,*)'kp1,kp2,kp3,kp4',kp1,kp2,kp3,kp4
!write(*,*)'kq1,kq2,kq3,kq4',kq1,kq2,kq3,kq4

pf=po+dp
qf=qo+dq

!write(*,*)'dp,dq',dp,dq
!write(*,*)'pf,qf',pf,qf
RETURN

END SUBROUTINE dqp_range_kutta





