PROGRAM triaxial
!-------------------------------------------------------------------------
! Program 6.11 Axisymmetric `undrained' strain of a 
!             solid using 8-node rectangular quadrilaterals
!             Undrained triaxial simulation; right half of sample.
!-------------------------------------------------------------------------
 USE main
 USE geom
 USE svd
 IMPLICIT NONE
 INTEGER,PARAMETER::iwp=SELECTED_REAL_KIND(15)
 INTEGER::fixed_freedoms,i,iel,incs,iy,k,ndim=2,nband=0,ndof=16,          &
   nels,neq,nip=4,nlen,nn,nod=8,nodof=2,nr,nst=4,nxe,nye,end,beginning,   &
   rate,max,ierr=-1,npri=1,nstep=200
 REAL(iwp)::cons_h,cons_v,det,dtim=0,pi,presc,jq,jp,aq,ap,lq,lp,          &
   sigmav_dssm,dshrpct,shrstrain,po,qo,pf,qf,penalty=1.e20_iwp,           &
   zero=.0_iwp,qf_sig,pf_sig,qo_sig,po_sig,pore_dssm
 CHARACTER(LEN=15)::argv,element='quadrilateral'
 LOGICAL::solid=.TRUE.
!-----------------------dynamic arrays------------------------------------
 INTEGER,ALLOCATABLE::g(:),g_g(:,:),g_num(:,:),kdiag(:),nf(:,:),no(:),    &
   node(:),num(:),sense(:),etype(:)
 REAL(iwp),ALLOCATABLE::bee(:,:),coord(:,:),ceetot(:,:),der(:,:),         &
 deriv(:,:),devp(:),delshr(:),eld_dssm(:),eps_dssm(:),fun(:),gc(:),       &
 g_coord(:,:),jac(:,:),km(:,:),tkm_dssm(:,:),dkv_dssm(:,:),new_dis(:),    &
 loads_dssm(:),points(:,:),qp_dssm(:,:),dkv_dssm_inv(:,:),sigma_dssm(:),  &
 storkv_dssm(:),etensor_dssm(:,:,:),totshr_dssm(:,:),delshr_dssm(:,:),    &
 weights(:),work_dssm(:,:),x_coords(:),y_coords(:),bdylds_dssm(:),        &
 bload_dssm(:),tload_dssm(:),tloads_dssm(:),dkv_dssm_sq(:,:),b1(:,:),     &
 b2(:,:),b3(:,:), teld_dssm(:)
!-----------------------input and initialisation--------------------------
 CALL getname(argv,nlen)
 call system_clock(beginning, rate,max)
 OPEN(10,FILE=argv(1:nlen)//'.dat')
 OPEN(11,FILE=argv(1:nlen)//'.res')
 OPEN(12,FILE=argv(1:nlen)//'.txt',action="write",status="replace")
 READ(10,*)nxe,nye,cons_h,cons_v
 READ(10,*)jq,jp,aq,ap,lq,lp
 CALL mesh_size(element,nod,nels,nn,nxe,nye)
 ALLOCATE(nf(nodof,nn),points(nip,ndim),weights(nip),g_coord(ndim,nn),    &
   x_coords(nxe+1),y_coords(nye+1),num(nod),                              &
   coord(nod,ndim),g_g(ndof,nels),fun(nod),                               &
   jac(ndim,ndim),der(ndim,nod),deriv(ndim,nod),         &
   g_num(nod,nels),bee(nst,ndof),km(ndof,ndof),etype(nels),devp(nst),     &
   g(ndof),gc(ndim),ceetot(nst,2),eps_dssm(nst),bload_dssm(ndof),         &
   sigma_dssm(nst),qp_dssm(2,1),eld_dssm(ndof),       &
   etensor_dssm(nst,nip,nels),delshr(nod),     &
   totshr_dssm(nip,nels),delshr_dssm(nip,nels),tkm_dssm(ndof,ndof),       &
   tload_dssm(ndof),b1(1,ndof),b2(2,ndof),b3(nst,ndof),teld_dssm(ndof))
 READ(10,*)x_coords,y_coords
 nf=1
 READ(10,*)nr,(k,nf(:,k),i=1,nr)
 CALL formnf(nf)
 neq=MAXVAL(nf)
 ALLOCATE(kdiag(neq),loads_dssm(neq),bdylds_dssm(neq),tloads_dssm(neq))
!-----------------------loop the elements to find global arrays sizes-----
 kdiag=0
 elements_1: DO iel=1,nels
   CALL geom_rect(element,iel,x_coords,y_coords,coord,num,'y')!returns element nodal cordinates and node numbers
   CALL num_to_g(num,nf,g) !returns steering vector g (the free nodes upto ndim per node, 0 if nodal freedome is fixed)
   g_num(:,iel)=num !num has element iel's node numbers in clockwise order
   g_coord(:,num)=TRANSPOSE(coord)
   g_g(:,iel)=g
   if(nband<bandwidth(g)) nband=bandwidth(g) !bandwidth is the max-min nodal freedom number for the element
   CALL fkdiag(kdiag,g)
 END DO elements_1
 CALL mesh(g_coord,g_num,argv,nlen,12)
 DO i=2,neq
   kdiag(i)=kdiag(i)+kdiag(i-1)
 END DO
 ALLOCATE(work_dssm(2*(nband+1)-1,neq),dkv_dssm(neq,2*(nband+1)-1))
 ALLOCATE(dkv_dssm_sq(neq,neq),dkv_dssm_inv(neq,neq),new_dis(neq))
 WRITE(11,'(2(A,I7))')                                                    &
   " There are",neq," equations and the skyline storage is",kdiag(neq)
!-----------------------calculate the ceetot matrix-----------------------
 pi=4.D0*DATAN(1.D0)
 CALL ceetotmat(ceetot,jq,jp,aq*pi/180.0,ap*pi/180.0)
 CALL sample(element,points,weights)
 etensor_dssm=zero
 dkv_dssm=zero
 tloads_dssm=zero
 new_dis=zero
 etensor_dssm(1,i,iel)=cons_v
 etensor_dssm(2,i,iel)=cons_h
 etensor_dssm(4,i,iel)=cons_h
 READ(10,*)fixed_freedoms
 IF(fixed_freedoms/=0)THEN
   ALLOCATE(node(fixed_freedoms),sense(fixed_freedoms),no(fixed_freedoms),&
     storkv_dssm(fixed_freedoms))
   READ(10,*)(node(i),sense(i),i=1,fixed_freedoms)
   DO i=1,fixed_freedoms
     no(i)=nf(sense(i),node(i))
   END DO
 END IF
 READ(10,*)incs,presc
 WRITE(11,'(/A)')"  step   disp      dev stress  hydr press  pore press"
 totshr_dssm=zero
 delshr_dssm=zero
 bdylds_dssm=zero
 loads_dssm=zero
 work_dssm=zero
 eld_dssm=zero
 teld_dssm=zero
 po_sig=(cons_v+cons_h)/2
 qo_sig=(cons_v-cons_h)/2
 WRITE(12,*)"  step   % disp      dev stress  pore press"
 WRITE(*,'(/A)')"  step  %_shear_strain      dev_stress  hyd_stress  pore_press"
!-----------------------displacement increment loop-----------------------
 disp_incs: DO iy=1,incs
   dkv_dssm=zero
   dkv_dssm_sq=zero
   dkv_dssm_inv=zero
   qp_dssm=zero
   dshrpct = presc/4*100
   shrstrain=iy*dshrpct
   WRITE(*,'(I5,4E12.4)')iy,shrstrain,qo_sig, po_sig, cons_h-(po_sig-qo_sig)
   CALL dqp_range_kutta(po_sig,qo_sig,shrstrain,dshrpct,jp,jq,lp,lq,ap,aq,&
   pf_sig,qf_sig)
   sigmav_dssm= (pf_sig-po_sig)+(qf_sig-qo_sig)
   po_sig=pf_sig
   qo_sig=qf_sig
   loads_dssm(no)= sigmav_dssm
   tloads_dssm= -(loads_dssm + bdylds_dssm)
!----------go round the Gauss points once to get stiffness matrix----------------
   elements_2: DO iel=1,nels
     g=g_g(:,iel)
     tkm_dssm=zero
     gauss_pts_2: DO i=1,nip
       CALL shape_fun(fun,points,i)
       CALL bee8(bee,coord,points(i,1),points(i,2),det)
       gc=MATMUL(fun,coord)
       bee(4,1:ndof-1:2)=fun(:)/gc(1)
       qp_dssm=zero
       qp_dssm(1,1)=((etensor_dssm(1,i,iel)+etensor_dssm(2,i,iel)) &
       /2)*EXP(-lp*totshr_dssm(i,iel))
       qp_dssm(2,1)=((etensor_dssm(1,i,iel)-etensor_dssm(2,i,iel)) &
       /2)*EXP(-lq*totshr_dssm(i,iel))
       b1(1,:)=bee(3,:)
       b2=MATMUL(qp_dssm,b1)
       b3=MATMUL(ceetot,b2)
       tkm_dssm=tkm_dssm+MATMUL(TRANSPOSE(bee),b3)*det*weights(i)*gc(1)
     END DO gauss_pts_2
   CALL formtb(dkv_dssm,tkm_dssm,g)
   END DO elements_2
   dkv_dssm(no,nband+1)=dkv_dssm(no,nband+1)+ penalty
   CALL sqtbl(dkv_dssm,nband,dkv_dssm_sq)
   CALL mpinv(dkv_dssm_sq, dkv_dssm_inv, ierr)
   new_dis=MATMUL(dkv_dssm_inv,tloads_dssm);
!-----------------------go round the Gauss points again------------------------
   elements_3: DO iel=1,nels
     bload_dssm=zero
     num=g_num(:,iel)
     coord=TRANSPOSE(g_coord(:,num))
     g=g_g(:,iel)
     eld_dssm=new_dis(g)
     teld_dssm=teld_dssm+eld_dssm
!------update the Gauss Point stresses and calculate pore pressures-------
     gauss_pts_3: DO i=1,nip
       CALL shape_fun(fun,points,i)
       CALL bee8(bee,coord,points(i,1),points(i,2),det)
       gc=MATMUL(fun,coord)
       delshr_dssm(i,iel)=DOT_PRODUCT(bee(3,:),eld_dssm)
       po=(etensor_dssm(1,i,iel)+etensor_dssm(2,i,iel))/2
       qo=(etensor_dssm(1,i,iel)-etensor_dssm(2,i,iel))/2
       dshrpct=delshr_dssm(i,iel)
       totshr_dssm(i,iel)=totshr_dssm(i,iel)+delshr_dssm(i,iel)
       shrstrain=totshr_dssm(i,iel)
       CALL dqp_range_kutta(po,qo,shrstrain,dshrpct,jp,jq,lp,lq,ap,aq,pf,qf)
       sigma_dssm(1)=pf+qf
       sigma_dssm(2)=pf-qf
       sigma_dssm(3)=qf
       sigma_dssm(4)=sigma_dssm(2)
       etensor_dssm(:,i,iel)=sigma_dssm
       pore_dssm=cons_h-etensor_dssm(2,i,iel)
       bload_dssm=bload_dssm+MATMUL(TRANSPOSE(bee),(etensor_dssm(:,i,iel)  &
       + pore_dssm))*det*weights(i)*gc(1)
     END DO gauss_pts_3
!-----------------------compute the total bodyloads vector ---------------
     bdylds_dssm(g)= bload_dssm
   END DO elements_3
 END DO disp_incs
 CALL dismsh_ensi(argv,nlen,nstep,nf,teld_dssm(1:))
 CALL mesh_ensi(argv,nlen,g_coord,g_num,element,etype,nf, &
 tloads_dssm(1:),nstep,npri,dtim,solid)
 call system_clock(end,rate,max)
 print *, "elapsed time: ",(end - beginning) / (rate),' secs.'
STOP
END PROGRAM triaxial